package raincoat;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.thrift7.TException;

import raincoat.api.RATreeNode;
import raincoat.builder.RaincoatFactory;
import raincoat.ratree.optimizers.OptimizeSelectOrdering;
import raincoat.sql.SqlParser;
import raincoat.sql.SqlScan;

import backtype.storm.Config;
import backtype.storm.LocalCluster;
import backtype.storm.StormSubmitter;
import backtype.storm.generated.AlreadyAliveException;
import backtype.storm.generated.InvalidTopologyException;
import backtype.storm.generated.Nimbus;
import backtype.storm.generated.NotAliveException;
import backtype.storm.generated.StormTopology;
import backtype.storm.utils.NimbusClient;
import backtype.storm.utils.Utils;

public class RaincoatTesting {

	public static void main(String[] args) {

		boolean local = true;
		long runtime = 30000;

		parseQuery("CREATE STREAM twitter " +
				"WITH FIELDS t_username, t_geotag, t_message, t_hashtag, t_mention " +
				"TYPE gen " +
				"OPTIONS every=500, seed=50, type=twitter");

		parseQuery("CREATE STREAM fb " +
				"WITH FIELDS f_username, f_message,  f_geotag, f_mention, f_link " +
				"TYPE gen " +
				"OPTIONS every=200, seed=0, type=fb");
		
		
		parseQuery("CREATE STREAM insta " +
	    "WITH FIELDS i_username, i_hashtags, i_geotag, "+
	    "i_picture, i_mention "+
	    "TYPE gen "+
	    "OPTIONS every=1000, seed=15, type=instagram'");

		parseQuery("CREATE STREAM linkedin " +
	    "WITH FIELDS l_username, l_message,  " +
	    "l_geotag, l_mention, l_link " +
	    "TYPE gen " +
	    "OPTIONS every=500, seed=23, type=fb");

		parseQuery("CREATE STREAM pin " +
	    "WITH FIELDS p_username, p_geotag, p_message, " +
	    "p_hashtag, p_mention "+
	    "TYPE gen "+
	    "OPTIONS every=1000, seed=50, type=twitter");

		String query = "select f_username " +
    "from twitter, fb, insta, linkedin, pin " +
    "where t_username = f_username " +
    "and f_username = i_username " +
    "and i_username = l_mention " +
    "and f_username = p_username " +
    "and f_mention = 'dummy' "+
    "and f_username = 'provocativelamentable' "+
    "and t_hashtag = 'beautiful' " +
    "every 4 seconds size 32";

		String topologyName = UUID.randomUUID().toString().split("-")[0];
		deployQuery(query, topologyName, local);

		Utils.sleep(30000);

		//		killTopology(topologyName);

	}


	public static RATreeNode parseQuery(String query) {
		SqlParser parser = new SqlParser(new SqlScan(query));
		return parser.parse();
	}

	private static void deployQuery(String query, String identifier, boolean local) {

		RATreeNode ratree = parseQuery(query);
		OptimizeSelectOrdering sel = new OptimizeSelectOrdering(null, true);
		ratree = sel.optimize(ratree);
		Config conf = getConfig();
		RaincoatFactory topologyFactory = new RaincoatFactory();

		StormTopology topology = topologyFactory.makeTopology(ratree);

		deployTopology(topology, identifier, conf, local);
		
	}

	public static void deployTopology(StormTopology topology,  
			String identifier, Config config, boolean local) {
		if (local) {
			config.setDebug(true);
			LocalCluster cluster = new LocalCluster();
			cluster.submitTopology(UUID.randomUUID().toString(), config, topology);
			return;
		}
		try {
			StormSubmitter.submitTopology(identifier, config, topology);
		} catch (AlreadyAliveException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidTopologyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	

	}


	public static void killTopology(String topology) {
		try {
			getNimbusClient().killTopology(topology);
		} catch (NotAliveException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private static Config __conf;
	private static Config getConfig() {
		if (__conf == null) {
			Config conf = new Config();
			conf.setDebug(true);
			conf.setNumWorkers(2);
			conf.put("topology.tick.tuple.freq.secs", 2);

			List<String> hostStrings = new ArrayList<String>();
			hostStrings.add("localhost");
			conf.put("kafka.hosts", hostStrings);
			conf.put("kafka.partitions", 1);
			conf.put("kafka.zkroot", "/kafka-spout");
			__conf = conf;
		}
		return __conf;
	}

	private static Nimbus.Client __client;

	private static Nimbus.Client getNimbusClient() {
		if (__client == null) {
			Map conf = Utils.readStormConfig();
			NimbusClient nimbusClient = NimbusClient.getConfiguredClient(conf);
			__client = nimbusClient.getClient();
		}
		return __client;
	}

}
