package raincoat.api;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;

import raincoat.api.OperationType;
import raincoat.storm.Tuples;

public interface Operation<T> extends Serializable {
	
	/**
	 * Initializer that should be called inside the prepare method of a Storm bolt
	 * @param conf
	 */
	void init(Map conf, Window<?> window);
	
	/**
	 * Performs the relational algebraic operation defined on implementation.
	 * @param tuples Input one or a list of tuples to perform operation on
	 * @return returns true if operation executed successfully and has output values, 
	 * else it returns false
	 */
	boolean performOperation(T tuples);
	
	/**
	 * Return the output of the current processed data. NB: The only values that is returned from
	 * this method is the ones that corresponds to the predefined fields. 
	 * @return a list of outputs based on the input passed into performOperation 
	 */
	List<Values> getOutputValues();
	
	/**
	 * Returns the fields of the current tuple. This method has to be static since it can only be defined once. 
	 * @return The fields of the tuples the operation outputs.
	 */
	Fields getFields();
	
	/**
	 * Defines the cardinality the operation input
	 * @return If operation only needs a single tuple SingleTupleOperation will be returned, if
	 * 		   one or more tuples can be processed by the current operation MultipleTuplesOperation
	 * 		   will be returned.
	 */
	OperationType getInputType();
	
	
	public Tuples getAnchoringTuples();
	
	 
	
}
