package raincoat.api;

import java.util.List;
import java.util.Set;

import backtype.storm.generated.Grouping;

import raincoat.sql.AlgebraOperator;

public interface RATreeNode {

	public String getIdentifier();
	/**
	 * returns the identifiers to the sources connected to the node
	 * @return
	 */
	public List<String> getSourceIdentifier();

	public AlgebraOperator getOperator(); 
	public List<RATreeNode> getChildren();

	/** Expected format for input tuples **/
	public List<String> getInputTuples();

	/** Outputted tuple format **/
	public List<String> getOutputTuples();

	public boolean containsField(String field);

	/**
	 * The cost of performing the operators in the RATreeNode 
	 * based on the number of incoming tuples
	 * @param numTuples number of incoming tuples
	 * @return the cost represented as a double
	 */
	public Double getCost(int numTuples); 

	public RATreeNode addChild(RATreeNode node);
	public boolean removeChild(RATreeNode node);
	public RATreeNode getParent();
	public void setParent(RATreeNode node);

	
	public boolean needsPostProcessing();
	public RATreeNode getPostProcessingNode();
	public void setPostProcessingNode(RATreeNode node);
	public Integer getParallelismHint(); 
	public void setParallelismHint(int parallelism);

	/** The grouping used on incoming streams **/
	public RaincoatGrouping getGrouping();
	
	
}
