package raincoat.api;

import java.io.Serializable;

import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;

/**
 * A stream is an abstraction of the stream passed between components of a topology.
 * The interface is designed to be applied in both bolt and spouts, but was first introduced
 * to support the implementation of a generic Spout, see {@link StreamSpout} for an example of
 * a Spout using stream as a source.  
 * 
 * 
 * @author gronnbeck
 *
 */
public interface Stream extends Serializable {
	
	/**
	 * A generic method for fetching and returning the next tuple in a defined stream
	 * @return the next tuple values in a stream
	 */
	Values nextTuple();
	
	/**
	 * A predefined list of String that defines the fields of a tuple inside a stream
	 * @return the fields of a tuple for the current stream
	 */
	Fields outputFields();
	
	/**
	 * Initializes the stream inside a Spout/Bolt. It is important to use this method when
	 * initializing structures that should be stored on the server running a Storm component.
	 * If not, the data structure will be initialized at the client side and then sent over the
	 * network to the server running the component.
	 *
	 * Also initializers may later be dependent on server configuration and topology context for
	 * further configuration purposes. 	
	 */
	void init();
	
	/**
	 * Returns a string representation of the stream. The name is used to map streams inputed
	 * in mist to existings streams. A stream must be 
	 * @return
	 */
	String getStreamName();
}
