package raincoat.api;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.thrift7.TException;

import backtype.storm.generated.ClusterSummary;
import backtype.storm.generated.ExecutorSummary;
import backtype.storm.generated.GlobalStreamId;
import backtype.storm.generated.Nimbus;
import backtype.storm.generated.NotAliveException;
import backtype.storm.generated.TopologyInfo;
import backtype.storm.generated.TopologySummary;
import backtype.storm.utils.NimbusClient;
import backtype.storm.utils.Utils;

public class TopologyStats {
	
	private String _topologyName;
	public TopologyStats(String topologyName) {
		_topologyName = topologyName;
	}
	
	private String __topologyID;
	public String getTopologyID() {
		if (__topologyID == null) {
			Nimbus.Client client = getNimbusClient();
			try {
				ClusterSummary clusterSumm = client.getClusterInfo();
				List<TopologySummary> topologySumms = clusterSumm.get_topologies();
				for (TopologySummary topologySumm : topologySumms) {
					if (topologySumm.get_name().equals(_topologyName)) {
						__topologyID = topologySumm.get_id();
						break;
					}
				}
				
			} catch (TException e) {} 
		}
		return __topologyID;
	}
	
	
	public Map getStats() {
		Nimbus.Client client = getNimbusClient();
		String topologyID = getTopologyID();
		List executeMsAvgs = getExecuteMsAvgs();
		System.err.println("Not implemented yet. " +
				"Should return a map with bot completion " +
				"and execution latency");
		return new HashMap();
	}
	
	public List<Map<String, Map<GlobalStreamId, Double>>> getExecuteMsAvgs() {
		List<Map<String, Map<GlobalStreamId, Double>>> executeMsAvgs 
			= new ArrayList<Map<String, Map<GlobalStreamId, Double>>>();
		
		Nimbus.Client client = getNimbusClient();
		String topologyID = getTopologyID();
		try {
			TopologyInfo topologyInfo = client.getTopologyInfo(topologyID);
			for (ExecutorSummary execSumm : topologyInfo.get_executors()) {
				execSumm.get_component_id();
				if (execSumm.get_stats().get_specific().is_set_bolt()) {
					Map<String, Map<GlobalStreamId, Double>> map 
						= (Map)execSumm.get_stats().get_specific()
							.get_bolt().get_execute_ms_avg();
					
					executeMsAvgs.add(map);
				}
			}
		} catch (NotAliveException e) {
		} catch (TException e) {
		}
		return executeMsAvgs;
	}
	
	private static Nimbus.Client __client;
	private static Nimbus.Client getNimbusClient() {
		if (__client == null) {
			Map conf = Utils.readStormConfig();
			makeNimbusClient(conf);
		}
		return __client;
	}
	
	private static Nimbus.Client makeNimbusClient(Map conf) {
		NimbusClient nimbusClient = NimbusClient.getConfiguredClient(conf);
		__client = nimbusClient.getClient();
		return __client;
	}
	
	public static void main(String[] args) {
		// get every "all-time" exec avgs from bolt component
		// to the default stream for topology "c0900ac"
		
		TopologyStats stats = new TopologyStats("c09e00ac");
		List<Map<String, Map<GlobalStreamId, Double>>> 
			execMsAvgs = stats.getExecuteMsAvgs();
		
		for (Map<String, Map<GlobalStreamId, Double>>
			execMsAvg : execMsAvgs) {
			Map<GlobalStreamId, Double> stat 
				= execMsAvg.get(":all-time");
			System.err.println(stat.values());
		}
		
	}

}
