package raincoat.api;

import java.io.Serializable;
import java.util.List;

import raincoat.storm.Tuples;

public interface Window<T> extends Serializable {
	/**
	 * Add a tuple to this Window. These tuples can be stored and used later by the current window.  
	 * @param tuple A tuple received from an ongoing stream 
	 */
	void addTuple(T tuple);

	  /**
	  * Evicts the expired tuples inside the current window. 
	  * @return a list of evicted tuples.
	  */
	  public Tuples evictExpiredTuples();
	
	/**
	 * Tuples accumulated by tumbling window can be retrieved using this method
	 * 
	 * @return All tuples inside the current window. If no tuples has been evicted
	 *  then a empty list should be returned.
	 */
	public Tuples getTuples();
	
	
	public Tuples getTuples(String condition);
	
	public List<String> getConditions();
	
	
	public void advanceWindow(); 
	
	/**
	 * Explicitly evict tuples
	 * 
	 * @param tuple
	 * @return
	 */
	public boolean expireTuple(T tuple);
	
}
