package raincoat.builder;

import java.util.List;

import backtype.storm.generated.GlobalStreamId;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.MessageId;
import backtype.storm.tuple.Tuple;
import backtype.storm.tuple.TupleImpl;
import backtype.storm.tuple.Values;

public class PipelineTuple implements Tuple {
	
	private Fields fields;
	private Values values;
	private String sourceComponentId;
	
	public PipelineTuple(String sourceComponentId, Fields fields, Values values) {
		this.fields = fields;
		this.values = values;
		this.sourceComponentId = sourceComponentId;
	}

	@Override
	public int size() {
		return fields.size();
	}

	@Override
	public int fieldIndex(String field) {
		return getFields().fieldIndex(field);
	}

	@Override
	public boolean contains(String field) {
		return getFields().contains(field);
	}

	@Override
	public Object getValue(int i) {
		return getValues().get(i);
	}

	@Override
	public String getString(int i) {
		return (String) getValues().get(i);
	}

	@Override
	public Integer getInteger(int i) {
		return (Integer) getValues().get(i);
	}

	@Override
	public Long getLong(int i) {
		return (Long) getValues().get(i);
	}

	@Override
	public Boolean getBoolean(int i) {
		return (Boolean) getValues().get(i);
	}

	@Override
	public Short getShort(int i) {
		return (Short) getValues().get(i);
	}

	@Override
	public Byte getByte(int i) {
		return (Byte) getValues().get(i);
	}

	@Override
	public Double getDouble(int i) {
		return (Double) getValues().get(i);
	}

	@Override
	public Float getFloat(int i) {
		return (Float) getValues().get(i);
	}

	@Override
	public byte[] getBinary(int i) {
		return (byte[]) getValues().get(i);
	}

	@Override
	public Object getValueByField(String field) {
		return getValues().get(fieldIndex(field));
	}

	@Override
	public String getStringByField(String field) {
		return (String) getValueByField(field);
	}

	@Override
	public Integer getIntegerByField(String field) {
		return (Integer) getValueByField(field);
	}

	@Override
	public Long getLongByField(String field) {
		return (Long) getValueByField(field);
	}

	@Override
	public Boolean getBooleanByField(String field) {
		return (Boolean) getValueByField(field);
	}

	@Override
	public Short getShortByField(String field) {
		return (Short) getValueByField(field);
	}

	@Override
	public Byte getByteByField(String field) {
		return (Byte) getValueByField(field);
	}

	@Override
	public Double getDoubleByField(String field) {
		return (Double) getValueByField(field);
	}

	@Override
	public Float getFloatByField(String field) {
		return (Float) getValueByField(field);
	}

	@Override
	public byte[] getBinaryByField(String field) {
		return (byte[]) getValueByField(field);
	}

	@Override
	public List<Object> getValues() {
		return this.values;
	}

	@Override
	public Fields getFields() {
		return this.fields;
	}

	@Override
	public List<Object> select(Fields selector) {
		return getFields().select(selector, values);
	}

	@Override
	public GlobalStreamId getSourceGlobalStreamid() {
		throw new IllegalStateException("Method not implemented in PipelineTuple");
	}

	@Override
	public String getSourceComponent() {
		return sourceComponentId;
	}

	@Override
	public int getSourceTask() {
		throw new IllegalStateException("Method not implemented in PipelineTuple");
	}

	@Override
	public String getSourceStreamId() {
		/* Since the tuple is a tuple belonging to a pipeline. 
		 *  We do just return a stream id named "__pipeline". 
		 *  Validation still throws a warning until we know this
		 *  implementation will not cause any errors. */
		return "__pipeline";
		
	}

	@Override
	public MessageId getMessageId() {
		throw new IllegalStateException("Method not implemented in PipelineTuple");	
	}

}
