package raincoat.contrib.redis;

import java.util.Map;
import java.util.concurrent.LinkedBlockingQueue;

import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;
import backtype.storm.utils.Utils;
import raincoat.api.Stream;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;
import redis.clients.jedis.JedisPubSub;

public class RedisStream implements Stream {

	private JedisPool pool;
	private String host;
	private int port;
	private String pattern;
	private LinkedBlockingQueue<String> queue;
	private String outputField;
	String streamName;
	
	public RedisStream(String streamName, String host, int port, String pattern, String outputField) {
		this.host = host;
		this.port = port;
		this.pattern = pattern;
		this.outputField = outputField;
	}
	
	class ListenerThread extends Thread {
		private LinkedBlockingQueue<String> queue;
		private JedisPool pool;
		public ListenerThread(LinkedBlockingQueue<String> queue, JedisPool pool) {
			this.queue = queue;
			this.pool = pool;
		}
		
		public void run() {
			JedisPubSub listener = new RedisSubscribeQueue(this.queue);
			Jedis jedis = this.pool.getResource();
			try {
				jedis.psubscribe(listener, pattern);
			} finally {
				pool.returnResource(jedis);
			}
		}

	}
	
	@Override
	public Values nextTuple() {
		String ret;
		while ((ret = queue.poll()) == null) {
			Utils.sleep(50);
		}
		return new Values(ret);
	}

	@Override
	public Fields outputFields() {
		return new Fields(this.outputField);
	}

	@Override
	public void init() {
		this.queue = new LinkedBlockingQueue<String>();
		this.pool = new JedisPool(new JedisPoolConfig(), this.host, this.port);
		ListenerThread listener = new ListenerThread(this.queue, this.pool);
		listener.start();
	}

	@Override
	public String getStreamName() {
		return this.streamName;	
	}


}
