package raincoat.operations;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Tuple;
import backtype.storm.tuple.Values;
import raincoat.api.Operation;
import raincoat.api.OperationType;
import raincoat.api.Window;
import raincoat.sql.Condition;
import raincoat.storm.Tuples;
import raincoat.windows.KeySlidingTimedWindow;

public class HashJoin implements Operation<Tuple> {

	private Fields _outfields;
	private List<Values> _outputList;
	private KeySlidingTimedWindow _window;
	private Tuples _anchoredTuples;
	private Condition _joinOnCondition;

	public HashJoin(Condition joinOn, Fields outfields) {
		_joinOnCondition = joinOn;
		// we assume that outfields are correct
		_outfields = outfields;
	}

	@Override
	public void init(Map conf, Window<?> window) {
		_outputList = new ArrayList<Values>();
		_window = (KeySlidingTimedWindow) window;
	}

	@Override
	public boolean performOperation(Tuple tuple) {
		Object key = keyForTuple(tuple);

		String condition = tuple.getSourceComponent();
		String oppositeCondition = _getOppositeConditionOf(condition);
		if (!_window.cacheContains(oppositeCondition, key)) {
			_window.addTuple(tuple);
			return false;
		}

		Tuples withTuples = _window.getTuples(oppositeCondition);
		joinTuple(tuple, withTuples);

		_anchoredTuples = new Tuples(tuple);
		_anchoredTuples.addAll(withTuples);

		return true;
	}

	private Object keyForTuple(Tuple tuple) {
		String fieldName = (String) _joinOnCondition.getFieldA().getValue();
		if (tuple.getFields().contains(fieldName))
			return tuple.getValueByField(fieldName);

		fieldName = (String) _joinOnCondition.getFieldB().getValue();
		return tuple.getValueByField(fieldName);

	}

	private String _getOppositeConditionOf(String condition) {

		List<String> conditions = new ArrayList<String>(_window.getConditions());
		conditions.remove(condition);
		if (conditions.size() == 0)
			return "";
		return conditions.get(0);
	}

	private void joinTuple(Tuple tuple, Tuples tuples) {
		for (Tuple aTuple : tuples) {
			// currently assuming that tuple A and B does not contain duplicate
			// fields
			Values outValues = getValuesForFields(aTuple, _outfields);
			outValues.addAll(getValuesForFields(tuple, _outfields));
			_outputList.add(outValues);
		}
	}

	private Values getValuesForFields(Tuple tuple, Fields fields) {
		Values outValues = new Values();
		for (String field : _outfields) {
			if (!tuple.contains(field))
				continue;

			Object val = tuple.getValueByField(field);
			outValues.add(val);
		}
		return outValues;
	}

	@Override
	public List<Values> getOutputValues() {
		List<Values> output = _outputList;
		_outputList = new ArrayList<Values>();
		return output;
	}

	@Override
	public Fields getFields() {
		return _outfields;
	}

	@Override
	public OperationType getInputType() {
		return OperationType.MultipleTuplesOperation;
	}

	@Override
	public Tuples getAnchoringTuples() {
		return _anchoredTuples;
	}

}
