package raincoat.operations;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import raincoat.api.Window;
import raincoat.sql.Condition;
import raincoat.sql.FieldValue;
import raincoat.sql.FieldValue.Type;
import raincoat.storm.Tuples;

import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Tuple;
import backtype.storm.tuple.Values;

/**
 * Most basic selection class. The field of a value has to be exactly
 * the same as the value it compares to. 
 * 
 * @author gronnbeck
 *
 */
@SuppressWarnings("serial")
public class Selection extends SingleTupleOperation {

    
    private List<Condition> conditions;
    private Fields outputFields;
    private List<Object> values;
    private Tuples anchoredTuples;
    
    
    public Selection(Fields outputFields, List<Condition> conditions) {
        this.conditions = conditions;
        this.outputFields = outputFields;
    }
    
    @Override
    public void init(Map conf, Window<?> window) {
        // TODO Auto-generated method stub
        
    }
    
    @Override
    public boolean performOperation(Tuple tuple) {
        boolean pass = true;
        for (Condition condition : this.conditions) {
            pass = pass && condition.compare(tuple); 

            if (!pass) break;
        }
        if (pass) {
            this.values = tuple.getValues();
            
            anchoredTuples = new Tuples();
            anchoredTuples.add(tuple);
            
        } 
        return pass;
    }

    @Override
    public List<Values> getOutputValues() {
        List<Values> out = new ArrayList<Values>();
        out.add(new Values(this.values.toArray()));
        return out;
    }

    @Override
    public Fields getFields() {
        return this.outputFields;
    }

    @Override
    public Tuples getAnchoringTuples() {
        return anchoredTuples;
    }
    

}
