package raincoat.operations;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import raincoat.api.Operation;
import raincoat.api.Window;
import raincoat.storm.Tuples;


import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Tuple;
import backtype.storm.tuple.Values;


@SuppressWarnings("serial")
public class Sum extends MultipleTuplesOperation  {

	private String field;
	private String outputField;
	private Values values;
	private Tuples anchoredTuples;
	
	/**
	 * Create a sum aggregating function for raincoat. The method assumes that 
	 * values of the fields provided as input is an {@link Integer}
	 * 
	 * @param field a string with the name of the field to sum. 
	 */
	public Sum(String field) {
		this(field, field);
	}
	
	public Sum(String field, String outputField) {
		this.field = field;
		this.outputField = outputField;
	}
	
	@Override
	public void init(Map conf, Window<?> window) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public boolean performOperation(Tuples tuples) {
		int sum = 0;
		for (Tuple tuple : tuples) {
			sum += tuple.getIntegerByField(this.field);
		}
		this.values = new Values(sum);
		
		anchoredTuples = new Tuples();
		anchoredTuples.addAll(tuples);
		
		return true;
	}

	@Override
	public List<Values> getOutputValues() {
		List<Values> out = new ArrayList<Values>();
		out.add(this.values);
		return out;
	}

	@Override
	public Fields getFields() {
		return new Fields(this.outputField);
	}

	@Override
	public Tuples getAnchoringTuples() {
		return anchoredTuples;
	}

}
