package raincoat.ratree;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import raincoat.api.RATreeNode;
import raincoat.sql.AlgebraOperator;
import raincoat.sql.FieldValue;


public class AggregatorNode extends Node {
	
	public enum AggregatorType {
		SUM,
		AVERAGE,
		COUNT,
		MIN,
		MAX
	}

	private AggregatorType type;
	private FieldValue field;

	public AggregatorType getType() {
		return type;
	}
	public FieldValue getField() {
		return field;
	}
	
	public AggregatorNode(List<String> outputTuples, AggregatorType type, FieldValue field) {
		super(AlgebraOperator.AGGREGATOR, outputTuples);
		this.type = type;
		this.field = field;
	}
	
	public List<String> getOutputTuples() {
		List<String> allOuts = super.getOutputTuples();
		List<String> outs = new ArrayList<String>(getInputTuples()); 
		for (String o : allOuts) {
			if (o.split(":").length > 1)
				outs.add(o);
			
		}
		return outs;
	}
	
	@Override
	public List<String> getSourceIdentifier() {
		List<String> sourceIds = new ArrayList<String>(); 
		List<RATreeNode> sources = getSources();
		
		for (RATreeNode node : sources) {
			if(node.containsField(field.getValue().toString().trim()))
				sourceIds.add(node.getIdentifier());
		}	

		return sourceIds;
	}
}
