package raincoat.ratree;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

import raincoat.api.CostModel;
import raincoat.api.RATreeNode;

public class JoinNodeComparator 
	implements Comparator<RATreeNode> {
	
	private Map<RATreeNode, Double> _cache;
	private RATreeCostModel _costModel;
	
	public JoinNodeComparator(RATreeCostModel costModel) {
		_cache = new HashMap<RATreeNode, Double>();
		_costModel = costModel;
	}

	@Override
	public int compare(RATreeNode o1, RATreeNode o2) {
		return _getJoinProbability(o1).compareTo(
				_getJoinProbability(o2));
	}
	
	private Double _getJoinProbability(RATreeNode node) {
		Map<RATreeNode, Double> cache = _getCache();
		if (!cache.containsKey(node)) {
			cache.put(node, _costModel.joinProbability(
					node.getSourceIdentifier().get(0), 
					node.getSourceIdentifier().get(1), 
					node));
		}
		return cache.get(node); 
	}
	
	private Map<RATreeNode, Double> _getCache() {
		return _cache;
	}
	
}