package raincoat.ratree;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import raincoat.api.RATreeNode;
import raincoat.sql.AlgebraOperator;
import raincoat.sql.Condition;
import raincoat.sql.FieldValue.Type;


public class SelectNode extends Node {
	
	private List<Condition> conditions;
	
	public List<Condition> getConditions() {
		if (conditions == null) conditions = new ArrayList<Condition>();
		return conditions;
	}

	public SelectNode(List<String> outputTuples, List<Condition> conditions) {
		super(AlgebraOperator.SELECT, outputTuples);
		this.conditions = conditions;
	}
	
	public SelectNode(String identifier, List<String> outputTuples, List<Condition> conditions) {
		super(identifier, AlgebraOperator.SELECT, outputTuples);
		this.conditions = conditions;
	}
	
	public List<String> getOutputTuples() {
		return getInputTuples();
	}

	@Override
	public List<String> getSourceIdentifier() {
		List<RATreeNode> sources = getSources();
		
		List<String> sourceIds = new ArrayList<String>();
		
		for (Condition condition : conditions) {
			String cond = condition.getFieldA().getType() == Type.TupleField ?
							condition.getFieldA().getValue().toString() :
							condition.getFieldB().getValue().toString();
			for (RATreeNode node : sources) {
				if(node.containsField(cond))
					sourceIds.add(node.getIdentifier());
			}	
		}
		return new ArrayList<String>(sourceIds);
	}
}
