package raincoat.ratree.optimizers;

import java.util.ArrayList;
import java.util.List;

import raincoat.RaincoatTesting;
import raincoat.api.RATreeNode;
import raincoat.api.TreeOptimizer;
import raincoat.ratree.SelectNode;
import raincoat.sql.Condition;
import raincoat.util.DFS;

public class MergeSelects implements TreeOptimizer {

	@Override
	public RATreeNode optimize(RATreeNode node) {
		MergeSelectsDFS merge = new MergeSelectsDFS();
		merge.traverserTree(node);
		return node;
	}
	
	
	private class MergeSelectsDFS extends DFS {

		@Override
		protected void execute(RATreeNode root) {
			// TODO: Is it really a good idea to not check if a selection operator selects from the same source? 
			if (!(root instanceof SelectNode)) return;
			SelectNode currentSelect = (SelectNode) root;
			
			if (!(currentSelect.getParent() 
					instanceof SelectNode)) return;
			
			SelectNode parent = 
					(SelectNode) currentSelect.getParent();
			
			if (!parent.getSourceIdentifier().get(0)
					.equalsIgnoreCase(
							currentSelect.getSourceIdentifier().get(0))) 
				return;
			
			List<RATreeNode> children = 
					new ArrayList<RATreeNode>(
							currentSelect.getChildren());
			
			List<Condition> currentConditions = 
					currentSelect.getConditions();
			
			parent.getConditions().addAll(currentConditions);
			parent.removeChild(currentSelect);
			for (RATreeNode child : children) {
				parent.addChild(child);
			}
			
		}
		
	}

}
