package raincoat.sql;

import java.io.Serializable;

import raincoat.sql.FieldValue.Type;

import backtype.storm.tuple.Tuple;

public class Condition implements Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = -874970469029324358L;
    private FieldValue a;
    private FieldValue b;
    private Operator operator;

    public FieldValue getFieldA() {
        return a;
    }

    public FieldValue getFieldB() {
        return b;
    }

    public Operator getOperator() {
        return operator;
    }
    //TODO refactor api, can it be done better?
    public Condition(FieldValue fieldA, FieldValue fieldB, Operator operator) {
        this.a = fieldA;
        this.b = fieldB;
        this.operator = operator;
    }
    
    public boolean isJoin() {
    	return getFieldA().getType() == Type.TupleField && getFieldB().getType() == Type.TupleField;
    }
    
    private boolean compare(Object obj1, Object obj2) {
        switch (operator) {
        case EQUAL:
            return obj1.equals(obj2);
        case GREATER_THAN:
            return (Integer)obj1 > (Integer)obj2;
        case GREATER_THAN_EQUAL:
            return (Integer)obj1 >= (Integer)obj2;
        case LESS_THAN:
            return (Integer)obj1 < (Integer)obj2;
        case LESS_THAN_EQUAL:
            return (Integer)obj1 <= (Integer)obj2;
        case NOT_EQUAL:
            return !obj1.equals(obj2);
        default:
            return false;
        }
    }
    
    public String toString() {
        return a.getValue().toString()+  "_"+ operator.toString() + "_"+ b.getValue().toString();
    }
    
    public enum Operator {  
        GREATER_THAN,
        GREATER_THAN_EQUAL,
        EQUAL,
        LESS_THAN,
        LESS_THAN_EQUAL,
        NOT_EQUAL;

        public static Operator fromString(String sval) {
            if(sval.equals(">"))        return GREATER_THAN;
            else if(sval.equals(">="))  return GREATER_THAN_EQUAL;
            else if(sval.equals("="))   return EQUAL;
            else if(sval.equals("<"))   return LESS_THAN;
            else if(sval.equals("<="))  return LESS_THAN_EQUAL;
            else if(sval.equals("!="))  return NOT_EQUAL;
            
            else throw new IllegalArgumentException("Operator not supported");
        }
    }

	public boolean compare(Tuple tuple) {
		Object valueA = valueForFieldValue(a, tuple);
		Object valueB = valueForFieldValue(b, tuple);
		return this.compare(valueA, valueB);
	}
	
	public boolean compare(Tuple tupleA, Tuple tupleB) {
		Object valueA = valueForFieldValue(a, tupleA);
		Object valueB = valueForFieldValue(b, tupleB);
		return this.compare(valueA, valueB);
	}
	
	private Object valueForFieldValue(FieldValue fieldValue, Tuple tuple) {
        Object obj = fieldValue.getValue();
        if (fieldValue.getType() == Type.TupleField) {
            String fieldName = (String)fieldValue.getValue();
            obj = tuple.getValueByField(fieldName);
        }
        return obj;
    }
}
