package raincoat.sql;

import java.io.Serializable;
import java.sql.Date;

public class FieldValue implements Serializable {

	private Type type;
	private Object value;
	
	public enum Type {
		TupleField,
		Integer,
		String,
		Date
	}
	
	public Type getType() {
		return this.type;
	}
	
	public Object getValue() {
		return this.value;
	}
	
	private static boolean isInt(Object value) {
		return Character.isDigit(value.toString().charAt(0));
	}
	
	private static boolean isDate(Object value) {
		try {
			Date.valueOf(value.toString());
			return true;
		} catch(IllegalArgumentException e) {
			return false;
		}
	}
	
	public static FieldValue fromString(String value) {
		if(isDate(value)) {
			Date date = Date.valueOf(value);
			return new FieldValue(Type.Date, date);
		} else {
			return new FieldValue(Type.String, value);
		}
	}
	
	public static FieldValue fromConstant(String value) {
		if(isInt(value)) {
			return new FieldValue(Type.Integer, Integer.parseInt(value.toString()));
		} else
			return new FieldValue(Type.TupleField, value);
	}
	
	public static FieldValue fromAggregator(String value) {
		return new FieldValue(Type.TupleField, value);
	}
	
	private FieldValue(Type type, Object value) {
		this.type = type;
		this.value = value;
	}
	
	public FieldValue(Object value, boolean isConstant) {
		this.value = value;
		
		if(isDate(value)) {
			
		}
		else if(isInt(value)) {
			this.value = Integer.parseInt(value.toString());
			this.type = Type.Integer;
		}
		else if(value instanceof String) {
			if (isConstant)
				type = Type.String;
			else
				type = Type.TupleField;
		}
 	}	
}
