package raincoat.sql;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.apache.commons.lang.StringUtils;

import raincoat.api.RATreeNode;
import raincoat.ratree.AggregatorNode;
import raincoat.ratree.JoinNode;
import raincoat.ratree.Node;
import raincoat.ratree.ProjectNode;
import raincoat.ratree.SelectNode;
import raincoat.ratree.SourceNode;
import raincoat.ratree.StaticJoinNode;
import raincoat.ratree.StaticSourceNode;
import raincoat.ratree.WindowNode;
import raincoat.ratree.AggregatorNode.AggregatorType;
import raincoat.sql.FieldValue.Type;

public class RATreeBuilder {
	
	private static List<SourceNode> sources = new ArrayList<SourceNode>();
	private static List<Node> nodes = new ArrayList<Node>();
	
	public static List<SourceNode> getSources() {
		return new ArrayList<SourceNode>(sources);
	}
	
	private static void createSources() {
		List<String> fields1  =new ArrayList<String>();
		fields1.add("a");
		
		SourceNode stream = new SourceNode(fields1, "streamA", "RedisStream", new HashMap<String, String>());
		sources.add(stream);
		
		List<String> fields2 = new ArrayList<String>();
		fields2.add("b");
		fields2.add("c");
		SourceNode stream2 = new SourceNode(fields2, "streamB", "RedisStream", new HashMap<String, String>());
		sources.add(stream2);
		
		List<String> fields3  =new ArrayList<String>();
		fields3.add("d");
		
		SourceNode stream3 = new SourceNode(fields3, "streamC", "RedisStream", new HashMap<String, String>());
		sources.add(stream3);
	}
	
	/**
	 * Search for the closest predecessor in the tree with the field
	 * @param fieldName Field that the predecessor must output
	 * @param except if not null, ignore this node in the search
	 * @return
	 */
	private static Node findPredecessor(String fieldName, Node except) {
		for(int i = nodes.size() - 1; i >= 0; i--) {
			if(nodes.get(i).containsField(fieldName)) {
				if(except == null)
					return nodes.get(i);
				else
					if(nodes.get(i) != except)
						return nodes.get(i);
			}
		}
		return null;
	}
	
	private static List<Node> getSources(List<String> from_sources) {
		ArrayList<Node> sourceList = new ArrayList<Node>();
		for (SourceNode source : sources) {
			for (String from : from_sources) {
				if(source.getName().equalsIgnoreCase(from))
					sourceList.add(source);
			}
		}
		
		return sourceList;
	}
	
	/**
	 * Search for the closest predecessor in the tree with the field
	 * @param fieldName Field that the predecessor must output
	 * @return
	 */
	private static Node findPredecestor(String fieldName) {
		return findPredecessor(fieldName, null);
	}
	
	public static void addSource(SourceNode source) {
		sources.add(source);
	}
	
	public static Node buildTree(List<String> projection_fields, List<String> from_sources, List<Condition> conditions, WindowNode window) {
		//TODO: temp, should be done by create stream statement
	//	createSources();
		List<Node> querySources = getSources(from_sources); 
		nodes.addAll(querySources);
		
	//	nodes.addAll(sources);
		buildJoinNodes(conditions);
		buildWhereNodes(conditions);
		projection_fields = replaceStar(projection_fields, querySources);
		buildAggregatorNodes(projection_fields);
		buildProjectNode(projection_fields);
		window.addChild(getLastNode());
		nodes.add(window);
		//debug
		printDebug(getLastNode());
		getLastNode().getSourceIdentifier();
		for (Node source : sources) {
			System.out.println(source.getIdentifier());
		}
		//--end debug
		
		return getLastNode();
	}

	private static List<String> replaceStar(List<String> projection_fields, List<Node> sources) {
		if(projection_fields.contains("*")) {
			projection_fields.remove("*");
			for (Node node : sources) {
				projection_fields.addAll(node.getOutputTuples());
			}
		}
		return projection_fields;
	}

	private static void buildProjectNode(List<String> projection_fields) {		
		Set<Node> childs = new HashSet<Node>();
		for (String field : projection_fields) {
			childs.add(findPredecestor(field));
		}
		ProjectNode project = new ProjectNode(projection_fields);
		
		List<String> inputTuples = new ArrayList<String>();
		for (Node child : childs) {
			project.addChild(child);
			inputTuples.addAll(child.getOutputTuples());
			
		}
		nodes.add(project);
	}

	private static void buildAggregatorNodes(List<String> projection_fields) {
		//add aggregators from projections:
		for (String field : projection_fields) {
			if(isAggregator(field)) {
				if(findPredecestor(field) == null)
					nodes.add(createAggregator(field));
			}
		}
	}

	private static void buildWhereNodes(List<Condition> conditions) {
		for (Condition condition : conditions) {
			//joins are already processed
			if(condition.isJoin())
				continue;
		
			ArrayList<Condition> c = new ArrayList<Condition>();
			c.add(condition);
			
			String field;
			//figure out whether it is field a or b that is a tuplefield
			if(condition.getFieldA().getType() == Type.TupleField)
				field = condition.getFieldA().getValue().toString();
			else
				field = condition.getFieldB().getValue().toString();
			
			// find the most recent predecessor in the current tree with the field
			Node pre = findPredecestor(field);
			
			if(isAggregator(field)) {
				if(pre == null) {
					// no existing aggregator in the tree. Create one:
					pre = createAggregator(field);
					nodes.add(pre);
				}
			}
			String id = UUID.randomUUID().toString();
			Node sel = new SelectNode("select_" + id, pre.getOutputTuples(), c);
			sel.addChild(pre);
			nodes.add(sel);
		}
	}

	private static void buildJoinNodes(List<Condition> conditions) {
		List<Condition> joins = getJoins(conditions);
		//add join nodes
		for (Condition condition : joins) {
			Node a = findPredecestor(condition.getFieldA().getValue().toString());
			Node b = findPredecessor(condition.getFieldB().getValue().toString(), a);
			List<String> inFields = new ArrayList<String>();
			inFields.addAll(a.getOutputTuples());
			inFields.addAll(b.getOutputTuples());
			Node join;
			if(isStaticSource(condition.getFieldA().getValue().toString())) {
				join = new StaticJoinNode(condition.toString(), condition, inFields); 
				join.addChild(a);
				System.err.println("static join" + condition.getFieldA().getValue());
			}
			else if(isStaticSource(condition.getFieldB().getValue().toString())) {
				join = new StaticJoinNode(condition.toString(), condition, inFields);
				join.addChild(b);
				System.err.println("static join" + condition.getFieldB().getValue());
			} else {
				join = new JoinNode(condition.toString(), condition, inFields);
				join.addChild(a);
				join.addChild(b);
				System.err.println("stream join [" + join.getIdentifier() + "]");
			}
			nodes.add(join);
		}
	}
	
	private static boolean isStaticSource(String fieldName) {	
		for (SourceNode source : sources) {
			if(!(source instanceof StaticSourceNode))
				continue;
			
			if(source.containsField(fieldName))
				return true;
		}
		return false;
	}
	
	private static Node getLastNode() {
		return nodes.get(nodes.size() - 1);
	}
	
	private static Node createAggregator(String fieldName) {
		String[] s = fieldName.split(":");
		Node pre = findPredecestor(s[1].trim());
		List<String> outputFields = new ArrayList<String>();
		outputFields.addAll(pre.getOutputTuples());
		outputFields.add(fieldName);
		Node aggregator = new AggregatorNode(outputFields, AggregatorType.valueOf(s[0].toUpperCase()), new FieldValue(s[1], false));
		aggregator.addChild(pre);
		return aggregator;
	}

	private static boolean isAggregator(String field) {
		String[] s = field.split(":");
		return (s.length > 1);
	}
	
	private static List<Condition> getJoins(List<Condition> conditions) {
		List<Condition> result = new ArrayList<Condition>();
		for (Condition condition : conditions) {
			//check if there is an implicit inner join.
			if(condition.isJoin())
				result.add(condition);
		}
		return result;
	}
	
	public static void printDebug(RATreeNode root) {
		printDebug(root, 0);
	}

	private static void printDebug(RATreeNode root, int lvl) {
		System.out.println(StringUtils.repeat("-", lvl) + "("+root.getIdentifier()+")"+ root + "source: " + root.getSourceIdentifier());
		for (RATreeNode child : root.getChildren()) {
			printDebug(child, lvl +2 );
		}
	}
}