package raincoat.sql;

import java.io.IOException;
import java.io.StreamTokenizer;
import java.io.StringReader;

public class SqlScan extends Tokenstream {
	private StreamTokenizer strtok;
        
    public SqlScan(String query) {
        strtok = new StreamTokenizer(new StringReader(query));
        setup();
        next();
    }
    
    private void setup() {
        strtok.resetSyntax();
        strtok.wordChars('a', 'z');
        strtok.wordChars('A', 'Z');
        strtok.wordChars('0', '9');
        strtok.wordChars('<', '>');
        strtok.wordChars('!', '!');
        strtok.wordChars('\'', '\'');
        strtok.wordChars('\"', '\"');
        strtok.wordChars('*', '*');
        strtok.wordChars('.', '.');
        strtok.wordChars('-', '-');
        strtok.wordChars('_', '_');
        strtok.whitespaceChars('(', ')');
        strtok.whitespaceChars(' ', ' ');
        strtok.whitespaceChars(',', ',');
        strtok.whitespaceChars('\t', '\t');
        strtok.whitespaceChars('\n', '\n');
        strtok.whitespaceChars('\r', '\r');
    }
    
    @Override
    public void next() {
        if(tok != EOF) {
            try {
                tok = strtok.nextToken();               
                switch(tok) {
                case StreamTokenizer.TT_WORD:
                    sval = strtok.sval.trim().toLowerCase();
                    if(sval.equals("select"))       tok = SELECT;
                    else if(sval.equals("from"))    tok = FROM;
                    else if(sval.equals("where"))   tok = WHERE;
                    else if(sval.equals("every"))   tok = EVERY;
                    else if(sval.equals("create"))  tok = CREATE;
                    else if(sval.equals("with"))  	tok = WITH;
                    else if(sval.equals("fields"))  tok = FIELDS;
                    else if(sval.equals("stream"))  tok = STREAM;
                    else if(sval.equals("static"))  tok = STATIC;
                    else if(sval.equals("type"))  	tok = TYPE;
                    else if(sval.equals("options")) tok = OPTIONS;
                    else if(sval.equals("date")) 	tok = DATE;
                    else if(sval.equals(">") ||
                            sval.equals(">=") ||
                            sval.equals("=") ||
                            sval.equals("<=") ||
                            sval.equals("!=") ||
                            sval.equals("<"))       tok = OPERATOR;
                    else if(sval.equals("and") ||
                            sval.equals("or"))      tok = OPERATION;
                    else if(sval.equals("seconds"))	tok = SECONDS;
                    else if(sval.equals("tuples"))	tok = TUPLES;
                	else if(sval.equals("size"))	tok = SIZE;
                    else if(sval.equals("sum") ||
                            sval.equals("average") ||
                            sval.equals("count") ||
                            sval.equals("min") ||
                            sval.equals("max"))     tok = AGGREGATOR;
                	else                            tok = NAME;
                    break;
                case StreamTokenizer.TT_EOF:
                    								tok = EOF;
                break;
                default: 
                    if((char)strtok.ttype == '\'' ||
                       (char)strtok.ttype == '\"')		tok = NAME;
                    else
                        throw new Tokenerror("Illegal token " + this);
                }
            } catch(IOException e) {
                throw new Tokenerror(e.getMessage());
            }
        }
    }
    
    @Override
    public String toString() {
        return this.sval;
    }
    
    
    class Tokenerror extends IllegalArgumentException {

        private static final long serialVersionUID = -3356424496048285471L;

        public Tokenerror(String s) {
            super(s);
        }
    }

}
