package raincoat.storm;

import java.util.Map;

import backtype.storm.task.OutputCollector;
import backtype.storm.task.TopologyContext;
import backtype.storm.topology.IRichBolt;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.tuple.Tuple;
import backtype.storm.tuple.Values;

import raincoat.api.Operation;
import raincoat.testsuit.performance.BoltStatistics;

public class FunctionBolt implements IRichBolt {

	/**
	 * serialVersioNUID is auto generated by eclipse
	 */
	private static final long serialVersionUID = 1L;

	private OutputCollector collector;

	private Operation<Tuple> singleTupleOperation;
	private BoltStatistics _boltStats;
	public FunctionBolt(Operation<Tuple> operation) {
		this.singleTupleOperation = operation;
	}

	@Override
	public void prepare(Map stormConf, TopologyContext context, OutputCollector collector) {
		this.collector = collector;
		// should not be null but NoWindow-class or something
		this.singleTupleOperation.init(stormConf, null);
		
		_boltStats = new BoltStatistics("func_" + context.getThisComponentId(), 
				""+context.getThisTaskId(), 5000);
		
	}

	@Override
	public void execute(Tuple input) {
		long before = System.currentTimeMillis();
		if (input.getSourceStreamId().equals("__tick")) {
			_boltStats.log();
			if(_boltStats.ready()) {
				_boltStats.setWindowSize(0);
				_boltStats.flush();
			}
			return;
		}
		if (singleTupleOperation.performOperation(input)) {
			for (Values output : singleTupleOperation.getOutputValues()) {
				collector.emit(input, output);
				_boltStats.incEmittedTuples(1);
			}
			
		}
		collector.ack(input);
		_boltStats.incAckedTuples(1);
		_boltStats.addExecTime(System.currentTimeMillis() - before);
	}

	@Override
	public void cleanup() {
		// TODO Auto-generated method stub

	}

	@Override
	public void declareOutputFields(OutputFieldsDeclarer declarer) {
		declarer.declare(singleTupleOperation.getFields());
	}

	@Override
	public Map<String, Object> getComponentConfiguration() {
		// TODO Auto-generated method stub
		return null;
	}
}
