package raincoat.storm;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.List;

import backtype.storm.task.OutputCollector;
import backtype.storm.task.TopologyContext;
import backtype.storm.topology.IRichBolt;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.tuple.Tuple;
import backtype.storm.tuple.Values;

import raincoat.api.Operation;
import raincoat.api.Window;
import raincoat.builder.PipelineTuple;
import raincoat.testsuit.performance.BoltStatistics;

public class WindowedBolt implements IRichBolt {

	/**
	 * serialVersioNUID is auto generated by eclipse
	 */
	private static final long serialVersionUID = 1L;

	private OutputCollector _collector;
	private BoltStatistics _stats;
	
	private Operation<Tuples> _operation;
	private Map<Operation<Tuples>, Window<Tuple>> _operationWindowMap;
	public WindowedBolt(Operation<Tuples> operation, Window<Tuple> window) {
		_operation = operation;
		_operationWindowMap = new HashMap<Operation<Tuples>, Window<Tuple>>();
		_operationWindowMap.put(operation,window);
		
	}
	
	@Override
	public void prepare(Map stormConf, TopologyContext context, OutputCollector collector) {
		_collector = collector;
		_stats = new BoltStatistics(context.getThisComponentId(), 
				""+context.getThisTaskId(), 
				10000);
	}

	@Override
	public void execute(Tuple input) {
		long before = System.currentTimeMillis();
		Window<Tuple> window = _operationWindowMap.get(_operation);
		if(input.getSourceStreamId().equals("__tick")) {
			Tuples tuples = window.getTuples();
			_operation.performOperation(tuples);
			
			for (Values output : _operation.getOutputValues()) {
				_collector.emit(_operation.getAnchoringTuples(), output);
			}
			
			window.advanceWindow();
			List<Tuple> evictedTuples = window.evictExpiredTuples();
			ack(evictedTuples);
			long execTime = System.currentTimeMillis() - before;
			_stats.addExecTime(execTime);
			_stats.incAckedTuples(evictedTuples.size());
			_stats.incEmittedTuples(_operation.getOutputValues().size());
			_stats.setWindowSize(tuples.size());
			_stats.log();
			if (_stats.ready()) _stats.flush();
			return;
		}
		
		window.addTuple(input);
		long execTime = System.currentTimeMillis() - before;
		_stats.addExecTime(execTime);
		
	}
	
	private void ack(List<Tuple> evictedTuples) {
		for (Tuple tuple : evictedTuples) {
			_collector.ack(tuple);
		}
	}
	

	@Override
	public void cleanup() {
		// TODO Auto-generated method stub

	}

	@Override
	public void declareOutputFields(OutputFieldsDeclarer declarer) {
		declarer.declare(_operation.getFields());
	}

	@Override
	public Map<String, Object> getComponentConfiguration() {
		// TODO Auto-generated method stub
		return null;
	}
}
