package raincoat.test;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import backtype.storm.Config;
import backtype.storm.LocalCluster;
import backtype.storm.StormSubmitter;
import backtype.storm.generated.AlreadyAliveException;
import backtype.storm.generated.InvalidTopologyException;
import backtype.storm.generated.StormTopology;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;
import raincoat.api.RATreeNode;
import raincoat.builder.PipelineTuple;
import raincoat.builder.RaincoatFactory;
import raincoat.ratree.JoinNode;
import raincoat.ratree.RATreeCostModel;
import raincoat.ratree.SelectNode;
import raincoat.ratree.SourceNode;
import raincoat.ratree.WindowNode;
import raincoat.ratree.WindowNode.WindowType;
import raincoat.sql.Condition;
import raincoat.sql.Condition.Operator;
import raincoat.sql.FieldValue;
import raincoat.storm.Tuples;

public class CostModelValidation {
	
	public static void main ( String[] args ) {
		
		RATreeNode window = nonOptimized();
		
		RaincoatFactory factory = new RaincoatFactory();
		StormTopology topology = factory.makeTopology(window);
		
		Tuples tuples = new Tuples();
		Random random = new Random();
		for (int i = 0; i < 1500; i++) {
			tuples.add(new PipelineTuple("S", 
					new Fields("i1", "i2"), 
					new Values(random.nextInt(10))));
			tuples.add(new PipelineTuple("T", 
					new Fields("j1", "j2"), 
					new Values(random.nextInt(10))));
			tuples.add(new PipelineTuple("U", 
					new Fields("u1", "u2"), 
					new Values(random.nextInt(10))));
			tuples.add(new PipelineTuple("V", 
					new Fields("v1", "v2"), 
					new Values(random.nextInt(10))));
		}
		
		RATreeCostModel costModel = new RATreeCostModel(tuples, 0.0001);
		double cost = costModel.costOf(window.getChildren().get(0));
		
		System.err.println("Cost of running the current topology: " + cost);
		
		Config config = new Config();
		config.setDebug(false);
		config.put("topology.tick.tuple.freq.secs", 2);
		
		LocalCluster localCluster = new LocalCluster();
		localCluster.submitTopology("test", config, topology);
		
		try {
			Thread.sleep(60000);
			localCluster.killTopology("test");
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		System.exit(0);
		
//		if(true)System.exit(0);
		try {
			StormSubmitter.submitTopology("test", config, topology);
		} catch (AlreadyAliveException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidTopologyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
	}
	
	private static RATreeNode nonOptimized() {
		Map<String, String> sourceOptions = new HashMap<String, String>();
		sourceOptions.put("from", "0");
		sourceOptions.put("to", "10");
		
		RATreeNode sourceS = new SourceNode("S", 
				new ArrayList<String>(new Fields("i1", "i2").toList()), 
				"S", "random-int", sourceOptions);
		
		
		RATreeNode sourceT = new SourceNode("T", 
				new ArrayList<String>(new Fields("j1", "j2").toList()), 
				"T", "random-int", sourceOptions);
		
		RATreeNode sourceU = new SourceNode("U", 
				new ArrayList<String>(new Fields("u1", "u2").toList()), 
				"U", "random-int", sourceOptions);
		
		RATreeNode sourceV = new SourceNode("V",  
				new ArrayList<String>(new Fields("v1", "v2").toList()), 
				"V", "random-int", sourceOptions);
		
		
		
		RATreeNode joinST = join("i1", "j1", sourceS, sourceT);
		RATreeNode joinTV = join("j1", "v1", sourceU, joinST);
		RATreeNode joinTU = join("j1", "u1", sourceV, joinTV);		

		RATreeNode selectS = select("i1", 0.2, joinTU.getOutputTuples());
		RATreeNode selectT = select("j1", 0.2, joinTU.getOutputTuples());
		RATreeNode selectV = select("v1", 0.2, joinTU.getOutputTuples());
		RATreeNode selectU = select("u1", 0.4, joinTU.getOutputTuples());
		
		RATreeNode window = new WindowNode(1, WindowType.SECONDS, 30);
		
		
		joinST.addChild(sourceS);
		joinST.addChild(sourceT);
		
		joinTV.addChild(sourceV);
		joinTV.addChild(joinST);
		
		joinTU.addChild(sourceU);
		joinTU.addChild(joinTV);
		
		selectS.addChild(joinTU);
		selectT.addChild(selectS);
		selectV.addChild(selectT);
		selectU.addChild(selectV);
		
		window.addChild(selectU);
		return window;
	}
	
	private static RATreeNode optimized() {
		Map<String, String> sourceOptions = new HashMap<String, String>();
		sourceOptions.put("from", "0");
		sourceOptions.put("to", "10");
		
		RATreeNode sourceS = new SourceNode("S",
				new ArrayList<String>(new Fields("i1", "i2").toList()), 
				"S", "random-int", sourceOptions);
		
		
		RATreeNode sourceT = new SourceNode("T",  
				new ArrayList<String>(new Fields("j1", "j2").toList()), 
				"T", "random-int", sourceOptions);
		
		RATreeNode sourceU = new SourceNode("U", 
				new ArrayList<String>(new Fields("u1", "u2").toList()), 
				"U", "random-int", sourceOptions);
		
		RATreeNode sourceV = new SourceNode("V", 
				new ArrayList<String>(new Fields("v1", "v2").toList()), 
				"V", "random-int", sourceOptions);
		
		
		RATreeNode selectS = select("i1", 0.2, sourceS.getOutputTuples());
		RATreeNode selectT = select("j1", 0.2, sourceT.getOutputTuples());
		RATreeNode selectV = select("v1", 0.2, sourceV.getOutputTuples());
		RATreeNode selectU = select("u1", 0.4, sourceU.getOutputTuples());
		
		RATreeNode joinST = join("i1", "j1", selectS, selectT);
		RATreeNode joinTV = join("v1", "j1", selectV, joinST);
		RATreeNode joinTU = join("u1", "v1", selectU, joinTV);		
		
		RATreeNode window = new WindowNode(1, WindowType.SECONDS, 30);
		
		
		joinST.addChild(selectS);
		joinST.addChild(selectT);
		
		joinTV.addChild(selectV);
		joinTV.addChild(joinST);
		
		joinTU.addChild(selectU);
		joinTU.addChild(joinTV);
		
		selectS.addChild(sourceS);
		selectT.addChild(sourceT);
		selectV.addChild(sourceV);
		selectU.addChild(sourceU);
		
		window.addChild(joinTU);
		return window;
	}
	
	
	private static RATreeNode select(String fieldName, double selectivity, 
			List<String> fields) {
		List<Condition> select1Conditions = new ArrayList<Condition>();
		select1Conditions.add(new Condition(
				new FieldValue(fieldName, false), 
				new FieldValue(((int)selectivity*10), true), 
				Operator.LESS_THAN_EQUAL));
		System.err.println(select1Conditions.get(0));
		RATreeNode select = new SelectNode( 
				fields, select1Conditions);
		return select;
	}
	
	
	private static RATreeNode join(String fieldS, String fieldT, 
			RATreeNode nodeS, RATreeNode nodeT) {
		List<String> inputTuples = new ArrayList<String>();
		List<String> outTuples = inputTuples;
		
		inputTuples.addAll(nodeS.getOutputTuples());
		inputTuples.addAll(nodeT.getOutputTuples());
		
		Condition condition =  new Condition(
				new FieldValue(fieldS, false),
				new FieldValue(fieldT, false), 
				Operator.EQUAL);
		
		return new JoinNode(condition, outTuples);
	}

}
