package raincoat.test;

import java.util.ArrayList;
import java.util.List;

import raincoat.api.Operation;
import raincoat.api.Window;
import raincoat.operations.HashJoin;
import raincoat.operations.Selection;
import raincoat.operations.Sum;
import raincoat.sql.Condition;
import raincoat.sql.Condition.Operator;
import raincoat.sql.FieldValue;
import raincoat.storm.FunctionBolt;
import raincoat.storm.HashJoinBolt;
import raincoat.storm.StatisticsBolt;
import raincoat.storm.Tuples;
import raincoat.storm.WindowedBolt;
import raincoat.test.generators.RandomIntegerSpout;
import raincoat.windows.SlidingTimedWindow;
import backtype.storm.Config;
import backtype.storm.LocalCluster;
import backtype.storm.generated.StormTopology;
import backtype.storm.topology.IRichSpout;
import backtype.storm.topology.TopologyBuilder;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Tuple;
import backtype.storm.utils.Utils;

public class RunTestTopologies {

	public static void main(String[] args) {

		Config conf = new Config();
		conf.setDebug(true);
		conf.setNumWorkers(2);
		conf.put("topology.tick.tuple.freq.secs", 1);

		LocalCluster cluster = new LocalCluster();
		cluster.submitTopology("test", conf, nextTestTopology());
		Utils.sleep(60000);
		cluster.killTopology("test");
		cluster.shutdown();

	}

	public static StormTopology nextTestTopology() {
		// RandomIntegerSpout{0,10}->SUM->OUT

		//		TopologyBuilder builder = new TopologyBuilder();
		//		builder.setSpout("source", new RandomIntegerSpout(0, 10));
		//
		//		List<Condition> conditions = new ArrayList<Condition>();
		//		conditions.add(new Condition(new FieldValue("integer", false), new FieldValue(5, true), Operator.GREATER_THAN_EQUAL));
		//		
		//		Operation<Tuple> operation = new Selection(new Fields("seq","integer"), conditions);
		//		builder.setBolt("select", new FunctionBolt(operation))
		//			.shuffleGrouping("source");
		//		
		//		
		//		List<Condition> conditions2 = new ArrayList<Condition>();
		//		conditions.add(new Condition(new FieldValue("integer", false), new FieldValue(6, true), Operator.GREATER_THAN_EQUAL));
		//		
		//		Operation<Tuple> operation2 = new Selection(new Fields("seq","integer"), conditions2);
		//		builder.setBolt("select2", new FunctionBolt(operation2))
		//			.shuffleGrouping("select");
		//		
		//			


		TopologyBuilder builder = new TopologyBuilder();
		RandomIntegerSpout spout1 =  new RandomIntegerSpout(0, 10, new Fields("integer_01", "integer_2"));
		RandomIntegerSpout spout2 = new RandomIntegerSpout(5, 15, new Fields("integer_11", "integer_12"));
		builder.setSpout("source1", spout1);
		builder.setSpout("source2", spout2);
		
		Condition joinCondition =  new Condition(
				new FieldValue(spout1.getFields().get(0), false), 
				new FieldValue(spout2.getFields().get(0), false),
				Operator.EQUAL);
		
		List<String> outListOfFields = new ArrayList<String>();
		outListOfFields.addAll(spout1.getFields().toList());
		outListOfFields.addAll(spout2.getFields().toList());
		
		Fields outfields = new Fields(outListOfFields);
		builder.setBolt("join", new HashJoinBolt(joinCondition, 
				outfields, "source1", "source2", 10, 2, 
				outfields))
				.shuffleGrouping("source2")
				.shuffleGrouping("source1");
		
		builder.setBolt("sum", new WindowedBolt(new Sum("integer_11"), 
				new SlidingTimedWindow(10, 2)))
				.shuffleGrouping("join");


		return builder.createTopology();
	}

}
