package raincoat.test;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import raincoat.api.RATreeNode;
import raincoat.builder.PipelineTuple;
import raincoat.ratree.RATreeCostModel;
import raincoat.ratree.SourceNode;
import raincoat.ratree.optimizers.OptimizeLeftDeepJoinOrder;
import raincoat.storm.Tuples;
import raincoat.test.generators.RandomIntegerSpout;
import raincoat.util.DFS;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;

public class TestDataFactory {
	
	public static RATreeCostModel makeCostModel(RATreeNode root) {
		ExtractSources extractSources = 
				new ExtractSources();
		
		extractSources.traverserTree(root);
		List<RATreeNode> sources = extractSources.getSources();
		
		Tuples tuples = new Tuples();
		Random random = new Random(System.currentTimeMillis());
		
		for (int i = 0; i < 1000; i++) {
			for (RATreeNode source : sources) {
				List<String> outputTuples = new ArrayList<String>();
				outputTuples.addAll(source.getOutputTuples());
				
				Values values = new Values();
				for (String field : outputTuples) {
					values.add(random.nextInt(10));
				}
				
				tuples.add(new PipelineTuple(source.getIdentifier(), 
							new Fields(outputTuples),  
							values));
			}
		}

		return new RATreeCostModel(tuples, 0.5);
	}
	
	private static class ExtractSources extends DFS {
		
		private List<RATreeNode> _sources;
		
		public ExtractSources() {
			_sources = new ArrayList<RATreeNode>();
		}
		
		
		public List<RATreeNode> getSources() {
			return _sources;
		}

		@Override
		protected void execute(RATreeNode root) {
			if (root instanceof SourceNode) {
				_sources.add(root);
			}
			
		}
		
	}
	
	
	public static void main(String[] args){
		
		OptimizeLeftDeepJoinOrder.query(
				"CREATE stream S " +
				"WITH FIELDS s1, s2 " +
				"TYPE randomint " +
				"OPTIONS from=0, to=10, rate=200");
		
		OptimizeLeftDeepJoinOrder.query(
				"CREATE stream T " +
				"WITH FIELDS t1, t2 " +
				"TYPE randomint " +
				"OPTIONS from=0, to=10, rate=200");
		
		
		RATreeNode query = OptimizeLeftDeepJoinOrder.query(
				"select s1 from S,T " +
				"where t1 = s1 " +
				"and s1 > 5 " +
				"every 2 seconds size 10");
		
		System.err.println(makeCostModel(query).costOf(
				query.getChildren().get(0)));
	}

}
