package raincoat.test.generators;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.UUID;

import backtype.storm.spout.SpoutOutputCollector;
import backtype.storm.task.TopologyContext;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.topology.base.BaseRichSpout;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;

public class RandomIntegerSpout extends BaseRichSpout {

	private int _from;
	public int _to;
	private Random _rand;
	private SpoutOutputCollector _collector;
	private long _lastUpdate;
	private Fields _outFields;
	private int _ackedTuples;
	private int _tuplesAckedSinceLastPrint;
	private int _rate;
	
	public RandomIntegerSpout(int from, int to, Fields outFields) {
		this(from, to, 1, outFields);
		
	}
	public RandomIntegerSpout(int from, int to, int rate, Fields outFields) {
		_from = from;
		_to = to;
		_ackedTuples = 0;
		_tuplesAckedSinceLastPrint = 0;
		_outFields = outFields;
		_rate = rate;
		
	}
	
	@Override
	public void open(Map conf, TopologyContext context,
			SpoutOutputCollector collector) {
		_rand = new Random();
		_collector = collector;
	}

	@Override
	public void nextTuple() {
		List<Integer> listOfValues = new ArrayList<Integer>();
		for (int i = 0; i < getFields().size(); i++) {
			int randomInteger = (_from + _rand.nextInt(_to) % _to);
			
			listOfValues.add(randomInteger);
			
		}
		Values values = new Values();
		values.addAll(listOfValues);
		_collector.emit(values, UUID.randomUUID());
		
    	try {
			Thread.sleep(_rate);
		} catch (InterruptedException e) {
		}	
		
	}

	@Override
	public void declareOutputFields(OutputFieldsDeclarer declarer) {
		declarer.declare(getFields());
		
	}
	
	
	public Fields getFields() {
		return _outFields;
	}
	
	@Override
	public void ack(Object tuple) {
		if (_lastUpdate==0) _lastUpdate = System.currentTimeMillis();
		_tuplesAckedSinceLastPrint++;
		if (System.currentTimeMillis() - _lastUpdate > 2000) {
			_lastUpdate = System.currentTimeMillis();
			_ackedTuples += _tuplesAckedSinceLastPrint;
			
			// Print Spout progress
			System.err.println("# tuples processed: " + _ackedTuples);
			System.err.println("# tuples since last: " + _tuplesAckedSinceLastPrint);
			
			_tuplesAckedSinceLastPrint = 0;
		}
	}
	


}
