package raincoat.testsuit.performance;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.LinkedList;
import java.util.Map;

public class BoltStatistics {
	
	private long _every;
	private long _last;
	private PrintWriter _writer;
	private LinkedList<String> _logCache;
	public BoltStatistics(String identifier, String task, int everyMs) {
	
		_every = everyMs;
		_last = System.currentTimeMillis();
		_logCache = new LinkedList<String>();
		
		Map<String,String> env = System.getenv();
		
		try {
			_writer = new PrintWriter(env.get("HOME") + "/logs/" + identifier + "_" + task + ".log");
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			System.exit(0);
		}
		
		_windowSize = 0;
		_ackedTuples = 0;
		_emittedTupels = 0;
		_avgExecTime = 0;
	}
	
	private long _windowSize;
	public void setWindowSize(long size) {
		_windowSize = size;
	}
	
	private long _ackedTuples;
	public void incAckedTuples(long with) {
		_ackedTuples += with;
	}
	
	private long _emittedTupels;
	public void incEmittedTuples(long with) {
		_emittedTupels += with;
	}
	
	private double _avgExecTime;
	private double _maxExecTime;
	private double _minExecTime;
	private double _totalExecTime;
	private int _execs;
	public void addExecTime(double execTime) {
		_execs++;
		if (_avgExecTime == 0) {
			_avgExecTime = execTime;
		} else {
			_avgExecTime = (_avgExecTime + execTime) / 2;
		}
		if (execTime > _maxExecTime) _maxExecTime = execTime;
		if (execTime < _minExecTime) _minExecTime = execTime;
		if (_minExecTime <= 0.0) _minExecTime = execTime;
		
		
		_totalExecTime += execTime;
	}
	
	
	
	
	public void log() {
		long currentMs = System.currentTimeMillis();
		_logCache.add(currentMs + "|" +
				_execs + "|" +
				_avgExecTime + "|" +
				_minExecTime + "|" +
				_maxExecTime + "|" +
				_totalExecTime + "|" +
				_windowSize + "|" + 
				_ackedTuples + "|" + 
				_emittedTupels + "\n");
		
		_avgExecTime = 0;
		_minExecTime = 0;
		_maxExecTime = 0;
		_execs = 0;
		_totalExecTime = 0;
		_ackedTuples = 0;
		_emittedTupels = 0;
		
	}
	
	public void flush() {
		long currentMs = System.currentTimeMillis();
		_last = currentMs;
		
		StringBuilder string = new StringBuilder();
		for (String line : _logCache) {
			string.append(line);
		}
		
		_writer.print(string);
		_writer.flush();
		
		_logCache = new LinkedList<String>();
	}
	
	public boolean ready() {
		long currentMs = System.currentTimeMillis();
		return (currentMs - _last >= _every); 
	}

}
