package raincoat.testsuit.performance;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Map;
import java.util.UUID;

import raincoat.RaincoatTesting;

import backtype.storm.spout.SpoutOutputCollector;
import backtype.storm.task.TopologyContext;
import backtype.storm.topology.OutputFieldsDeclarer;
import backtype.storm.topology.base.BaseRichSpout;
import backtype.storm.tuple.Fields;
import backtype.storm.tuple.Values;
import backtype.storm.utils.Utils;

public class DataGenSpout extends BaseRichSpout {
	
	private DataGenerator datagen;
	private SpoutOutputCollector collector;
	private long everyMS;
	private Fields out;
	private PrintWriter _writer;
	
	public DataGenSpout(DataGenerator datagen, Fields out, 
			long everyMS) {
		this.datagen = datagen;
		this.everyMS = everyMS;
		this.out = out;
	}

	@Override
	public void open(Map conf, TopologyContext context,
			SpoutOutputCollector collector) {
		this.collector = collector;
		
		Map<String, String> getenv = System.getenv();
		try {
			_writer = new PrintWriter(getenv.get("HOME") + 
					"/logs/spout_" + context.getThisComponentId() + 
					"_" + context.getThisTaskId() + ".log");
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		
	}

	@Override
	public void nextTuple() {
		if (datagen.hasNext()) {
			Values values = datagen.next();
			collector.emit(values, UUID.randomUUID().toString());
			emitted++;
		}
		
		PerformanceTestSuit.sleep(everyMS);
		
		log();
		
	}

	@Override
	public void declareOutputFields(OutputFieldsDeclarer declarer) {
		declarer.declare(this.out);
	}
	
	@Override
	public void ack(Object id) {
		acked++;
		
		log();
	}
	
	@Override
	public void fail(Object id) {
		failed++;
	}
	
	private long lastLogged;
	private long acked;
	private long failed;
	private long emitted;
	private void log() {
		int logEveryMS = 5000;
		long current = System.currentTimeMillis();
		if (lastLogged == 0) lastLogged = System.currentTimeMillis();
		
		if (current - lastLogged >= logEveryMS) {
			
			_writer.println(current +
					"|" + acked +
					"|" + failed +
					"|" + emitted);
			_writer.flush();
			
			acked = 0;
			failed = 0;
			emitted = 0;
			lastLogged = current;
		}
	}

}
