package raincoat.testsuit.performance;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Random;

import backtype.storm.tuple.Values;

public abstract class LoadDataGenerator implements DataGenerator {

	Random rand;
	String paragraph;
	List<String> usernames;
	List<String> hashtags;
	public LoadDataGenerator(long seed) {
		rand = new Random(seed);
		usernames = getUsernames();
		paragraph = getParagraph();
		hashtags = getHashtags();
	}

	public List<String> getUsernames() {

		if (usernames == null) {
			usernames = new ArrayList<String>();
			Map<String, String> getenv = System.getenv();
			String path = getenv.get("HOME");
			try {
				FileReader reader = new FileReader(path + "/data/400usernames.txt");
				BufferedReader br = new BufferedReader(reader);
				String username;
				while ((username = br.readLine()) != null) {
					usernames.add(username);
				}
				br.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
		}
		return usernames;
	}

	public String getParagraph() {

		if (paragraph == null) {
			paragraph = "";
			Map<String, String> getenv = System.getenv();
			String path = getenv.get("HOME");
			try {
				FileReader reader = new FileReader(path + "/data/slipsum.txt");
				BufferedReader br = new BufferedReader(reader);
				String potParagraph = br.readLine();
				if (potParagraph != null) paragraph = potParagraph;
				br.close();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return paragraph;
	}
	
	public List<String> getHashtags() {
		if (hashtags == null) {
			hashtags = new ArrayList<String>();
			Map<String, String> getenv = System.getenv();
			String path = getenv.get("HOME");
			try {
				FileReader reader = new FileReader(path + "/data/hashtags.txt");
				BufferedReader br = new BufferedReader(reader);
				String tag;
				while ((tag = br.readLine()) != null) {
					hashtags.add(tag);
				}
				br.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return hashtags;
	}

	public String getMessage() {
		int lenght = rand.nextInt(150);
		int start = rand.nextInt(paragraph.length() - lenght);
		String message = paragraph.substring(start, start + lenght);
		return message;
	}


	public static void main (String[] args) {
		//		LoadDataGenerator test = new LoadDataGenerator() {
		//			
		//			@Override
		//			public Values next() {
		//				// TODO Auto-generated method stub
		//				return null;
		//			}
		//			
		//			@Override
		//			public boolean hasNext() {
		//				// TODO Auto-generated method stub
		//				return false;
		//			}
		//			
		//			@Override
		//			public void forceStartIndex(int i) {
		//				// TODO Auto-generated method stub
		//				
		//			}
		//		};
		//		
		//		System.err.println(test.getUsernames().size());
		//		System.err.println(test.getParagraph());
	}

}
