package raincoat.testsuit.performance;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import backtype.storm.Config;
import backtype.storm.generated.StormTopology;

import raincoat.RaincoatTesting;
import raincoat.api.RATreeNode;
import raincoat.builder.RaincoatFactory;
import raincoat.ratree.JoinNode;
import raincoat.util.DFS;

public class ParallelismTestSuit {
	
	public static void main(String[] args){
		
		int parallelism = Integer.parseInt(args[0]);
		int runTime = Integer.parseInt(args[1]);
		int rate1 = Integer.parseInt(args[2]);
		int rate2 = Integer.parseInt(args[3]);
		
		List<String> queries = new ArrayList<String>();
		queries.add("CREATE STREAM twitter " +
				"WITH FIELDS t_username, t_geotag, t_message, t_hashtag, t_mention " +
				"TYPE gen " +
				"OPTIONS every="+rate1+", seed=50, type=twitter");
		
		queries.add("CREATE STREAM fb " +
				"WITH FIELDS f_username, f_message, f_mention, f_geotag f_link " +
				"TYPE gen " +
				"OPTIONS every="+rate2+", seed=0, type=fb");
		
		queries.add("SELECT t_username " +
				"FROM twitter, fb " +
				"WHERE f_username = t_username " +
				"EVERY 2 SECONDS SIZE 30");
		
		RaincoatTesting.parseQuery(queries.get(0));
		RaincoatTesting.parseQuery(queries.get(1));
		
		RATreeNode queryTree = RaincoatTesting.parseQuery(queries.get(2));
		ApplyParallelismHint parallelismHint = new ApplyParallelismHint(parallelism);
		parallelismHint.traverserTree(queryTree);
		
		RaincoatFactory factory = new RaincoatFactory();
		StormTopology topology = factory.makeTopology(queryTree);
		String identifier = UUID.randomUUID().toString().split("-")[0];
		Config config = new Config();
		config.put("topology.tick.tuple.freq.secs", 2);
		config.put(Config.TOPOLOGY_MESSAGE_TIMEOUT_SECS, 120);
		RaincoatTesting.deployTopology(topology, identifier, config, false);
		
		
		PerformanceTestSuit.sleep(runTime);
		RaincoatTesting.killTopology(identifier);
	}
	
	public static class ApplyParallelismHint extends DFS {

		private int phint;
		public ApplyParallelismHint(int parallelismHint) {
			phint = parallelismHint;
		}
		@Override
		protected void execute(RATreeNode root) {
			if (root instanceof JoinNode) root.setParallelismHint(phint);
		}
		
	}

}
