package raincoat.windows;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.Condition;

import backtype.storm.tuple.Tuple;
import raincoat.api.Window;
import raincoat.storm.Tuples;
import raincoat.storm.TuplesHash;

public class KeySlidingTimedWindow implements Window<Tuple> {

	private Map<String, TuplesHash> _caches;
	private WindowCondition _condition;
	
	public KeySlidingTimedWindow(String[] conditions, WindowCondition condition, int maxAge, int tickSize) {
		_caches = new HashMap<String, TuplesHash>();
		for (String c : conditions) {
			_caches.put(c, new TuplesHash(maxAge, tickSize));
		}
		_condition = condition;
	}

	@Override
	public void addTuple(Tuple tuple) {
		TuplesHash cache = _condition.getCache(_caches, tuple);
		cache.cache(_condition.getKey(tuple), tuple);
	}

	@Override
	public Tuples evictExpiredTuples() {
		Tuples evictedTuples = new Tuples();
		for (TuplesHash subCache : _caches.values()) {
			for (Window<?> window : subCache.values()) {
				evictedTuples.addAll(window.evictExpiredTuples());
			}
		}
		return evictedTuples;
	}

	@Override
	public Tuples getTuples() {
		Tuples tuples = new Tuples();
		for (TuplesHash subCache : _caches.values()) {
			for (Window<?> window : subCache.values()) {
				tuples.addAll(window.getTuples());
			}
		}
		return tuples;
	}

	@Override
	public Tuples getTuples(String condition) {
		TuplesHash cache = _caches.get(condition);
		Tuples tuples = new Tuples();
		for (Window <Tuple> window : cache.values()) {
			tuples.addAll(window.getTuples());
		}
		return tuples;
	}

	@Override
	public List<String> getConditions() {
		return new ArrayList<String>(_caches.keySet());
	}

	@Override
	public void advanceWindow() {
		for (TuplesHash subCache : _caches.values()) {
			for (Window<?> window : subCache.values()) {
				window.advanceWindow();
			}
		}
	}

	@Override
	public boolean expireTuple(Tuple tuple) {
		// TODO Auto-generated method stub
		return false;
	}
	
	public boolean cacheContains(String condition, Object key) {
		TuplesHash cache = _caches.get(condition);
		return cache.containsKey(key);
	}

}
