package raincoat.windows;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import backtype.storm.tuple.Tuple;
import raincoat.api.Window;
import raincoat.storm.Tuples;

public class SlidingTimedWindow implements Window<Tuple> {

	private static final long serialVersionUID = 2347599800749884522L;

	private LinkedList<Tuples> _cache;
	private Tuples _expiredTuples;
	private int _maxAge;
	private int _tickSize;
	private int _currentTick;

	public SlidingTimedWindow(int maxAge, int tickSize) {
		_cache = new LinkedList<Tuples>();
		_cache.addLast(new Tuples());
		_maxAge = maxAge;
		_tickSize = tickSize;
		_expiredTuples = new Tuples();
	}

	@Override
	public void addTuple(Tuple tuple) {
		Tuples tuples = _cache.getLast();
		tuples.add(tuple);
	}

	private int getAge() {
		return _currentTick * _tickSize;
	}

	@Override
	public void advanceWindow() {
		_currentTick++;
		_cache.addLast(new Tuples());
	}

	@Override
	public Tuples evictExpiredTuples() {
		if (getAge() >= _maxAge) {
			moveWipeThreshold();
			Tuples expiredTuples = _cache.poll();
			expiredTuples.addAll(_expiredTuples);
			for (Tuple tuple : _expiredTuples) {
				for (Tuples tuples : _cache) {
					tuples.remove(tuple);
				}
			}
			_expiredTuples = new Tuples();
			return expiredTuples;
		}
		return new Tuples();
	}

	private void moveWipeThreshold() {
		_currentTick = 0;
	}

	@Override
	public Tuples getTuples() {
		List<Tuple> allTuples = new ArrayList<Tuple>();
		for (Tuples ts : _cache) {
			for (Tuple t : ts) {
				allTuples.add(t);
			}
		}
		return new Tuples(allTuples);
	}

	@Override
	public boolean expireTuple(Tuple tuple) {
		for (Tuples tuples : _cache) {
			if (tuples.contains(tuple)) {
				tuples.remove(tuple);

				return true;
			}
		}
		return false;
	}

	@Override
	public Tuples getTuples(String condition) {
		return getTuples();
	}

	@Override
	public List<String> getConditions() {
		return new ArrayList<String>();
	}


}
